var is = require('unist-util-is');

var nodeToString = require('hast-util-to-string');

var strip = require('strip-indent');

var _require = require('lodash'),
    flow = _require.flow;

var codeFromNode = require('./code-from-node');

function getComponentName(value) {
  var match = value.match(/^\<\\?(\w+)/);
  return match && match[1];
}

function removePlaygroundTag(code) {
  var open = codeFromNode(function (p) {
    return p.isJSXOpeningElement() && p.node.name.name === 'PlayGround';
  });
  var close = codeFromNode(function (p) {
    return p.isJSXClosingElement() && p.node.name.name === 'PlayGround';
  });
  return code.replace(open(code), '').replace(close(code), '');
}

function trim(arg) {
  return arg.trim();
}

function addCodeProp(node, index) {
  var name = getComponentName(node.value);
  var tagOpen = new RegExp("^\\<".concat(name));

  if (name === 'PlayGround') {
    var code = flow(nodeToString, removePlaygroundTag, strip, trim)(node);
    node.value = node.value.replace(tagOpen, "<".concat(name, " __position={").concat(index, "} __code={`").concat(code, "`}"));
  }
}

module.exports = function rehypePlayground() {
  return function (tree) {
    var reactNodes = tree.children.filter(function (node) {
      return is('jsx', node);
    }).map(addCodeProp);
  };
};