var logger = require('@drawbotics/logger');

var is = require('unist-util-is');

var visit = require('unist-util-visit');

var find = require('unist-util-find');

var GithubSlugger = require('github-slugger');

function getTitle(heading) {
  var text = find(heading, function (n) {
    return is('text', n);
  });
  return text ? text.value : null;
}

function getUrl(heading) {
  var slugger = new GithubSlugger();
  return slugger.slug(getTitle(heading));
}

function buildEntries(headings, depth) {
  return headings.map(function (heading) {
    if (heading.__done) {
      return null;
    } else if (heading.depth === depth) {
      heading.__done = true; // mark it as done

      return {
        title: getTitle(heading),
        url: getUrl(heading)
      };
    } else if (heading.depth > depth) {
      return {
        entries: buildEntries(headings, heading.depth)
      };
    } else {
      return null;
    }
  }).filter(Boolean);
}

module.exports = function toc() {
  return function (tree) {
    var headings = [];
    visit(tree, 'heading', function (n) {
      return headings.includes(n) ? null : headings.push(n);
    });
    var entries = buildEntries(headings, 1);
    var tocNode = visit(tree, function (node) {
      if (is('html', node) && node.value.match(/<Toc\s/)) {
        node.value = node.value.replace(/<Toc\s/, "<Toc __entries={".concat(JSON.stringify(entries), "} "));
      }
    });
  };
};